package com.ekexiu.portal.dao;

import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.orm.annotation.dao.DAO;
import org.jfw.apt.orm.annotation.dao.method.From;
import org.jfw.apt.orm.annotation.dao.method.operator.DeleteWith;
import org.jfw.apt.orm.annotation.dao.method.operator.Insert;
import org.jfw.apt.orm.annotation.dao.method.operator.SelectList;
import org.jfw.apt.orm.annotation.dao.method.operator.Update;
import org.jfw.apt.orm.annotation.dao.method.operator.UpdateWith;
import org.jfw.apt.orm.annotation.dao.param.Set;
import org.jfw.util.PageQueryResult;

import com.ekexiu.portal.po.Image;
import com.ekexiu.portal.po.Operation;
import com.ekexiu.portal.po.Professor;
import com.ekexiu.portal.po.Resource;

@DAO
public abstract class OperationDao {
	@Autowrie
	private ImageDao imageDao;
	
	public ImageDao getImageDao() {
		return imageDao;
	}

	public void setImageDao(ImageDao imageDao) {
		this.imageDao = imageDao;
	}

	@Insert
	public abstract int insert(Connection con, Operation operation) throws SQLException;
	
	@Update
	public abstract int update(Connection con, Operation operation) throws SQLException;
	
	@UpdateWith
	@From(Operation.class)
	public abstract int updateDetailDemand(Connection con, String operationId, @Set String detailDemand) throws SQLException;
	
	@UpdateWith
	@From(Operation.class)
	public abstract int updateOperationStatus(Connection con, String operationId, @Set int operationStatus) throws SQLException;
	
	@UpdateWith
	@From(Operation.class)
	public abstract int updateDeliverTime(Connection con, String operationId, @Set String deliverTime) throws SQLException;
	
	@UpdateWith
	@From(Operation.class)
	public abstract int updateDealTime(Connection con, String operationId, @Set String dealTime) throws SQLException;
	
	@UpdateWith
	@From(Operation.class)
	public abstract int updatePayMethod(Connection con, String operationId, @Set String payMethod) throws SQLException;
	
	/**
	 * 查询有多少资源合作正在进行
	 * @param con
	 * @param professorId 登陆者ID
	 * @return 返回正在合作的资源申请数量
	 * @throws SQLException
	 */
	public int queryOnGoingOper(Connection con, @Nullable String professorId) throws SQLException {
		String sql = " SELECT COUNT(1) FROM OPERATION O WHERE (OPERATION_STATUS IN (1,2,3,7)) "
				+ " AND (O.RESOURCE_ID IN (SELECT R.RESOURCE_ID FROM RESOURCE R WHERE R.PROFESSOR_ID = ?) "
				+ " OR O.DEMAND_ID = ?) ";
		PreparedStatement ps = con.prepareStatement(sql);
		ps.setString(1, professorId);
		ps.setString(2, professorId);
		ResultSet rs = ps.executeQuery();
		rs.next();
		int count = rs.getInt(1);
		return count;
	}
	
	public Operation query(Connection con,String operationId) throws SQLException{
        int index = 1;
        String sql = "SELECT REPLY_NOTES,OPERATION.CREATE_TIME,OPERATION_ID,OPERATION_STATUS,"
        		+ " DEMAND_ID,DETAIL_DEMAND,DELIVER_TIME,OPERATION.RESOURCE_ID,DEAL_PRICE,DEAL_TIME,"
        		+ " PAY_METHOD,P1.NAME,RESOURCE_NAME,RESOURCE.PROFESSOR_ID,P2.NAME,APPLYSQUARE "
        		+ " FROM OPERATION INNER JOIN PROFESSOR P1 ON DEMAND_ID = P1.ID "
        		+ " INNER JOIN RESOURCE ON OPERATION.RESOURCE_ID = RESOURCE.RESOURCE_ID "
        		+ " INNER JOIN PROFESSOR P2 ON RESOURCE.PROFESSOR_ID = P2.ID "
        		+ " WHERE OPERATION_ID = ?";
        PreparedStatement ps = con.prepareStatement(sql);
        try{
            ps.setString(index++,operationId);
            ResultSet rs = ps.executeQuery();
            try{
                if(rs.next()){
                    Operation operation = new Operation();
                    operation.setReplyNotes(rs.getString(1));
                    operation.setCreateTime(rs.getString(2));
                    operation.setOperationId(rs.getString(3));
                    operation.setOperationStatus(rs.getInt(4));
                    Professor professor = new Professor();
                    professor.setId(rs.getString(5));
                    operation.setDetailDemand(rs.getString(6));
                    operation.setDeliverTime(rs.getString(7));
                    Resource resource = new Resource();
                    resource.setResourceId(rs.getString(8));
                    operation.setDealPrice(rs.getBigDecimal(9));
                    operation.setDealTime(rs.getString(10));
                    operation.setPayMethod(rs.getString(11));
                    professor.setName(rs.getString(12));
                    operation.setProfessor(professor);
                    resource.setResourceName(rs.getString(13));
                    Professor professor2 = new Professor();
                    professor2.setId(rs.getString(14));
                    professor2.setName(rs.getString(15));
                    resource.setProfessor(professor2);
                    operation.setResource(resource);
                    operation.setApplysquare(rs.getString(16));
                    return operation;
                }else{
                    return null;
                }
            }finally{
                try{rs.close();}catch(Exception e1){}
            }
        }finally{
            try{ps.close();}catch(Exception e2){}
        }
    }
	
	public List<Operation> query(Connection con) throws SQLException{
        String sql = "SELECT REPLY_NOTES,OPERATION.CREATE_TIME,OPERATION_ID,OPERATION_STATUS,"
        		+ " DEMAND_ID,DETAIL_DEMAND,DELIVER_TIME,OPERATION.RESOURCE_ID,DEAL_PRICE,DEAL_TIME,"
        		+ " PAY_METHOD,P1.NAME,RESOURCE_NAME,RESOURCE.PROFESSOR_ID,P2.NAME,APPLYSQUARE "
        		+ " FROM OPERATION INNER JOIN PROFESSOR P1 ON DEMAND_ID = P1.ID "
        		+ " INNER JOIN RESOURCE ON OPERATION.RESOURCE_ID = RESOURCE.RESOURCE_ID "
        		+ " INNER JOIN PROFESSOR P2 ON RESOURCE.PROFESSOR_ID = P2.ID";
        PreparedStatement ps = con.prepareStatement(sql);
        try{
            ResultSet rs = ps.executeQuery();
            try{
                List<Operation> operations = new ArrayList<Operation>();
                while(rs.next()){
                    Operation operation =  new Operation();
                    operation.setReplyNotes(rs.getString(1));
                    operation.setCreateTime(rs.getString(2));
                    operation.setOperationId(rs.getString(3));
                    operation.setOperationStatus(rs.getInt(4));
                    Professor professor = new Professor();
                    professor.setId(rs.getString(5));
                    operation.setDetailDemand(rs.getString(6));
                    operation.setDeliverTime(rs.getString(7));
                    Resource resource = new Resource();
                    resource.setResourceId(rs.getString(8));
                    operation.setDealPrice(rs.getBigDecimal(9));
                    operation.setDealTime(rs.getString(10));
                    operation.setPayMethod(rs.getString(11));
                    professor.setName(rs.getString(12));
                    operation.setProfessor(professor);
                    resource.setResourceName(rs.getString(13));
                    Professor professor2 = new Professor();
                    professor2.setId(rs.getString(14));
                    professor2.setName(rs.getString(15));
                    resource.setProfessor(professor2);
                    operation.setResource(resource);
                    operation.setApplysquare(rs.getString(16));
                    operations.add(operation);
                }
                return operations;
            }finally{
                try{rs.close();}catch(Exception e1){}
            }
        }finally{
            try{ps.close();}catch(Exception e2){}
        }
    }
	
	/**
	 * 搜索当前用户的所有资源申请（包括申请别人和别人申请自己的）
	 * @param con
	 * @param id 登录用户ID
	 * @param pageSize
	 * @param pageNo
	 * @return 返回所有资源申请记录（包括申请别人和别人申请自己的）
	 * @throws SQLException
	 */
	public PageQueryResult<Operation> queryPro(Connection con, @Nullable String id,int pageSize,int pageNo) throws SQLException{
        int total = 0;
        PageQueryResult<Operation> queryResult = new PageQueryResult<Operation>();
        int index = 1;
        boolean hasId = null != id;
        StringBuilder sql = new StringBuilder();
        if(hasId){
            sql.append(" WHERE O.RESOURCE_ID IN (SELECT R.RESOURCE_ID FROM RESOURCE R WHERE R.PROFESSOR_ID = ?) OR (O.DEMAND_ID = ?) ");
        }
        StringBuilder whereSql = sql;
        sql = new StringBuilder();
        sql.append(" SELECT COUNT(1) FROM OPERATION O ");
        if(whereSql.length()>0){
            sql.append(whereSql);
        }
        PreparedStatement ps = con.prepareStatement(sql.toString());
        try{
            if(hasId){
                ps.setString(index++,id);
                ps.setString(index++,id);
            }
            queryResult.setPageSize(pageSize);
            ResultSet rs = ps.executeQuery();
            try{
                rs.next();
                total = rs.getInt(1);
            }finally{
                try{rs.close();}catch(Exception e1){}
            }
        }finally{
            try{ps.close();}catch(Exception e2){}
        }
        queryResult.setTotal(total);
        if(0== total){
        	queryResult.setPageNo(1);
        	queryResult.setData(Collections.<Operation>emptyList());
            return queryResult;
        }
        index = 1;
        boolean firstPage = (1 == pageNo);
        if(firstPage){
        	queryResult.setPageNo(1);
            sql = new StringBuilder();
            sql.append("SELECT REPLY_NOTES,APPLYSQUARE,OPERATION_ID,OPERATION_STATUS,DEMAND_ID,"
            		+ " DETAIL_DEMAND,DELIVER_TIME,O.RESOURCE_ID,DEAL_PRICE,DEAL_TIME,PAY_METHOD,"
            		+ " O.CREATE_TIME,R.RESOURCE_NAME,SUPPORTED_SERVICES,R.DESCP,R.PROFESSOR_ID,P1.NAME,P2.NAME "
            		+ " FROM OPERATION O LEFT JOIN RESOURCE R ON O.RESOURCE_ID = R.RESOURCE_ID "
            		+ " LEFT JOIN PROFESSOR P1 ON DEMAND_ID = P1.ID "
            		+ " LEFT JOIN PROFESSOR P2 ON R.PROFESSOR_ID = P2.ID");
            if(whereSql.length()>0){
                sql.append(whereSql);
            }
            sql.append(" ORDER BY O.CREATE_TIME DESC ");
            sql.append(" LIMIT ").append(pageSize);
        }else{
            int pageNum = total / pageSize;
            if(total % pageSize != 0){
                ++pageNum;
            }
            if(pageNo > pageNum){
                pageNo = pageNum;
            }
            queryResult.setPageNo(pageNo);
            --pageNo;
            int offset = (pageNo * pageSize);
            sql = new StringBuilder();
            sql.append("SELECT REPLY_NOTES,APPLYSQUARE,OPERATION_ID,OPERATION_STATUS,DEMAND_ID,"
            		+ " DETAIL_DEMAND,DELIVER_TIME,O.RESOURCE_ID,DEAL_PRICE,DEAL_TIME,PAY_METHOD,"
            		+ " O.CREATE_TIME,R.RESOURCE_NAME,SUPPORTED_SERVICES,R.DESCP,R.PROFESSOR_ID,P1.NAME,P2.NAME "
            		+ " FROM OPERATION O LEFT JOIN RESOURCE R ON O.RESOURCE_ID = R.RESOURCE_ID "
            		+ " LEFT JOIN PROFESSOR P1 ON DEMAND_ID = P1.ID "
            		+ " LEFT JOIN PROFESSOR P2 ON R.PROFESSOR_ID = P2.ID");
            if(whereSql.length()>0){
                sql.append(whereSql);
            }
            sql.append(" ORDER BY O.CREATE_TIME DESC ");
            sql.append(" LIMIT ").append(pageSize).append(" OFFSET ").append(offset);
        }
        ps = con.prepareStatement(sql.toString());
        try{
            if(hasId){
                ps.setString(index++,id);
                ps.setString(index++,id);
            }
            ResultSet rs = ps.executeQuery();
            try{
                List<Operation> operations = new ArrayList<Operation>();
                queryResult.setData(operations);
                int size = 0;
                while((size<pageSize) && rs.next()){
                    ++size;
                    Operation operation =  new Operation();
                    String replyNotes = rs.getString(1);
                    if(rs.wasNull()){
                    	replyNotes = null;
                    }
                    operation.setReplyNotes(replyNotes);
                    String applysqare = rs.getString(2);
                    if(rs.wasNull()){
                    	applysqare = null;
                    }
                    operation.setApplysquare(applysqare);
                    operation.setOperationId(rs.getString(3));
                    operation.setOperationStatus(rs.getInt(4));
                    Professor professor = new Professor();
                    professor.setId(rs.getString(5));
                    operation.setDetailDemand(rs.getString(6));
                    operation.setDeliverTime(rs.getString(7));
                    Resource resource = new Resource();
                    resource.setResourceId(rs.getString(8));
                    BigDecimal dealPrice = rs.getBigDecimal(9);
                    if(rs.wasNull()){
                    	dealPrice = null;
                    }
                    operation.setDealPrice(dealPrice);
                    String dealTime = rs.getString(10);
                    if(rs.wasNull()){
                    	dealTime = null;
                    }
                    operation.setDealTime(dealTime);
                    String payMethod = rs.getString(11);
                    if(rs.wasNull()){
                    	payMethod = null;
                    }
                    operation.setPayMethod(payMethod);
                    operation.setCreateTime(rs.getString(12));
                    resource.setResourceName(rs.getString(13));
                    resource.setSupportedServices(rs.getString(14));
                    resource.setDescp(rs.getString(15));
                    List<Image> images = this.imageDao.queryRes(con, resource.getResourceId());
                    resource.setImages(images);
                    Professor professor2 = new Professor();
                    professor2.setId(rs.getString(16));
                    professor.setName(rs.getString(17));
                    operation.setProfessor(professor);
                    professor2.setName(rs.getString(18));
                    resource.setProfessor(professor2);
                    operation.setResource(resource);
                    operations.add(operation);
                }
                return queryResult;
            }finally{
                try{rs.close();}catch(Exception e3){}
            }
        }finally{
            try{ps.close();}catch(Exception e4){}
        }
    }
	
	/**
	 * 分页按资源申请者ID查资源申请
	 * @param con
	 * @param demandId 资源申请者ID
	 * @param pageSize 每页记录数
	 * @param pageNo 当前页码
	 * @return 返回该用户ID申请的所有资源
	 * @throws SQLException
	 */
	public PageQueryResult<Operation> queryDemPage(Connection con,String demandId,int pageSize,int pageNo) throws SQLException{
        int total = 0;
        PageQueryResult<Operation> queryResult = new PageQueryResult<Operation>();
        int index = 1;
        String sql = null;
        sql = "  WHERE OPERATION.DEMAND_ID = ?        ";
        String whereSql = sql;
        sql = "SELECT COUNT(1) FROM OPERATION";
        sql = sql + whereSql;
        PreparedStatement ps = con.prepareStatement(sql);
        try{
            ps.setString(index++,demandId);
            queryResult.setPageSize(pageSize);
            ResultSet rs = ps.executeQuery();
            try{
                rs.next();
                total = rs.getInt(1);
            }finally{
                try{rs.close();}catch(Exception e1){}
            }
        }finally{
            try{ps.close();}catch(Exception e2){}
        }
        queryResult.setTotal(total);
        if(0== total){
        	queryResult.setPageNo(1);
        	queryResult.setData(Collections.<Operation>emptyList());
            return queryResult;
        }
        index = 1;
        boolean firstPage = (1 == pageNo);
        if(firstPage){
        	queryResult.setPageNo(1);
            sql = "SELECT REPLY_NOTES,OPERATION.CREATE_TIME,APPLYSQUARE,OPERATION_ID,OPERATION_STATUS,"
            		+ " DEMAND_ID,DETAIL_DEMAND,DELIVER_TIME,OPERATION.RESOURCE_ID,DEAL_PRICE,DEAL_TIME,"
            		+ " PAY_METHOD,P1.NAME,RESOURCE_NAME,RESOURCE.PROFESSOR_ID,P2.NAME "
            		+ " FROM OPERATION INNER JOIN PROFESSOR P1 ON DEMAND_ID = P1.ID "
            		+ " INNER JOIN RESOURCE ON OPERATION.RESOURCE_ID = RESOURCE.RESOURCE_ID "
            		+ " INNER JOIN PROFESSOR P2 ON RESOURCE.PROFESSOR_ID = P2.ID";
            sql = sql + whereSql;
            sql = sql + "ORDER BY OPERATION.CREATE_TIME DESC";
            sql = sql + " LIMIT " + pageSize;
        }else{
            int pageNum = total / pageSize;
            if(total % pageSize != 0){
                ++pageNum;
            }
            if(pageNo > pageNum){
                pageNo = pageNum;
            }
            queryResult.setPageNo(pageNo);
            --pageNo;
            int offset = (pageNo * pageSize);
            sql = "SELECT REPLY_NOTES,OPERATION.CREATE_TIME,APPLYSQUARE,OPERATION_ID,OPERATION_STATUS,"
            		+ " DEMAND_ID,DETAIL_DEMAND,DELIVER_TIME,OPERATION.RESOURCE_ID,DEAL_PRICE,DEAL_TIME,"
            		+ " PAY_METHOD,P1.NAME,RESOURCE_NAME,RESOURCE.PROFESSOR_ID,P2.NAME "
            		+ " FROM OPERATION INNER JOIN PROFESSOR P1 ON DEMAND_ID = P1.ID "
            		+ " INNER JOIN RESOURCE ON OPERATION.RESOURCE_ID = RESOURCE.RESOURCE_ID "
            		+ " INNER JOIN PROFESSOR P2 ON RESOURCE.PROFESSOR_ID = P2.ID";
            sql = sql + whereSql;
            sql = sql + "ORDER BY OPERATION.CREATE_TIME DESC";
            sql = sql + " LIMIT " + pageSize+ " OFFSET "+ offset;
        }
        ps = con.prepareStatement(sql);
        try{
            ps.setString(index++,demandId);
            ResultSet rs = ps.executeQuery();
            try{
                List<Operation> operations = new ArrayList<Operation>();
                queryResult.setData(operations);
                int size = 0;
                while((size<pageSize) && rs.next()){
                    ++size;
                    Operation operation =  new Operation();
                    String replyNotes = rs.getString(1);
                    if(rs.wasNull()){
                    	replyNotes = null;
                    }
                    operation.setReplyNotes(replyNotes);
                    operation.setCreateTime(rs.getString(2));
                    String applysquare = rs.getString(3);
                    if(rs.wasNull()){
                    	applysquare = null;
                    }
                    operation.setApplysquare(applysquare);
                    operation.setOperationId(rs.getString(4));
                    operation.setOperationStatus(rs.getInt(5));
                    Professor professor = new Professor();
                    professor.setId(rs.getString(6));
                    operation.setDetailDemand(rs.getString(7));
                    operation.setDeliverTime(rs.getString(8));
                    Resource resource = new Resource();
					resource.setResourceId(rs.getString(9));
                    BigDecimal dealPrice = rs.getBigDecimal(10);
                    if(rs.wasNull()){
                    	dealPrice = null;
                    }
                    operation.setDealPrice(dealPrice);
                    String dealTime = rs.getString(11);
                    if(rs.wasNull()){
                    	dealTime = null;
                    }
                    operation.setDealTime(dealTime);
                    String payMethod = rs.getString(12);
                    if(rs.wasNull()){
                    	payMethod = null;
                    }
                    operation.setPayMethod(payMethod);
                    professor.setName(rs.getString(13));
                    operation.setProfessor(professor);
					resource.setResourceName(rs.getString(14));
					Professor professor2 = new Professor();
                    professor2.setId(rs.getString(15));
                    professor2.setName(rs.getString(16));
                    resource.setProfessor(professor2);
					operation.setResource(resource);
					operations.add(operation);
                }
                return queryResult;
            }finally{
                try{rs.close();}catch(Exception e3){}
            }
        }finally{
            try{ps.close();}catch(Exception e4){}
        }
    }
	
	/**
	 * 分页按资源ID查资源申请，返回资申请人姓名和资源名称
	 * @param con
	 * @param resourceId 资源ID
	 * @param pageSize 每页记录数
	 * @param pageNo 当前页码
	 * @return 返回当前资源ID下的所有资源申请记录
	 * @throws SQLException
	 */
	public PageQueryResult<Operation> queryResPage(Connection con,String resourceId,int pageSize,int pageNo) throws SQLException{
        int total = 0;
        PageQueryResult<Operation> queryResult = new PageQueryResult<Operation>();
        int index = 1;
        String sql = null;
        sql = " WHERE OPERATION.RESOURCE_ID = ? ";
        String whereSql = sql;
        sql = "SELECT COUNT(1) FROM OPERATION";
        sql = sql + whereSql;
        PreparedStatement ps = con.prepareStatement(sql);
        try{
            ps.setString(index++,resourceId);
            queryResult.setPageSize(pageSize);
            ResultSet rs = ps.executeQuery();
            try{
                rs.next();
                total = rs.getInt(1);
            }finally{
                try{rs.close();}catch(Exception e1){}
            }
        }finally{
            try{ps.close();}catch(Exception e2){}
        }
        queryResult.setTotal(total);
        if(0== total){
        	queryResult.setPageNo(1);
        	queryResult.setData(Collections.<Operation>emptyList());
            return queryResult;
        }
        index = 1;
        boolean firstPage = (1 == pageNo);
        if(firstPage){
        	queryResult.setPageNo(1);
            sql = "SELECT REPLY_NOTES,OPERATION.CREATE_TIME,APPLYSQUARE,OPERATION.OPERATION_ID,OPERATION_STATUS,"
            		+ " DEMAND_ID,DETAIL_DEMAND,DELIVER_TIME,OPERATION.RESOURCE_ID,DEAL_PRICE,DEAL_TIME,"
            		+ " PAY_METHOD,P1.NAME,RESOURCE_NAME,RESOURCE.PROFESSOR_ID,P2.NAME "
            		+ " FROM OPERATION INNER JOIN PROFESSOR P1 ON DEMAND_ID = P1.ID "
            		+ " INNER JOIN RESOURCE ON OPERATION.RESOURCE_ID = RESOURCE.RESOURCE_ID "
            		+ " INNER JOIN PROFESSOR P2 ON RESOURCE.PROFESSOR_ID = P2.ID ";
            sql = sql + whereSql;
            sql = sql + "ORDER BY OPERATION.CREATE_TIME DESC";
            sql = sql + " LIMIT " + pageSize;
        }else{
            int pageNum = total / pageSize;
            if(total % pageSize != 0){
                ++pageNum;
            }
            if(pageNo > pageNum){
                pageNo = pageNum;
            }
            queryResult.setPageNo(pageNo);
            --pageNo;
            int offset = (pageNo * pageSize);
            sql = "SELECT REPLY_NOTES,OPERATION.CREATE_TIME,APPLYSQUARE,OPERATION.OPERATION_ID,OPERATION_STATUS,"
            		+ " DEMAND_ID,DETAIL_DEMAND,DELIVER_TIME,OPERATION.RESOURCE_ID,DEAL_PRICE,DEAL_TIME,"
            		+ " PAY_METHOD,P1.NAME,RESOURCE_NAME,RESOURCE.PROFESSOR_ID,P2.NAME "
            		+ " FROM OPERATION INNER JOIN PROFESSOR P1 ON DEMAND_ID = P1.ID "
            		+ " INNER JOIN RESOURCE ON OPERATION.RESOURCE_ID = RESOURCE.RESOURCE_ID "
            		+ " INNER JOIN PROFESSOR P2 ON RESOURCE.PROFESSOR_ID = P2.ID";
            sql = sql + whereSql;
            sql = sql + "ORDER BY OPERATION.CREATE_TIME DESC";
            sql = sql + " LIMIT " + pageSize+ " OFFSET "+ offset;
        }
        ps = con.prepareStatement(sql);
        try{
            ps.setString(index++,resourceId);
            ResultSet rs = ps.executeQuery();
            try{
                List<Operation> operations = new ArrayList<Operation>();
                queryResult.setData(operations);
                int size = 0;
                while((size<pageSize) && rs.next()){
                    ++size;
                    Operation operation =  new Operation();
                    String replyNotes = rs.getString(1);
                    if(rs.wasNull()){
                    	replyNotes = null;
                    }
                    operation.setReplyNotes(replyNotes);
                    operation.setCreateTime(rs.getString(2));
                    String applyaquare = rs.getString(3);
                    if(rs.wasNull()){
                    	applyaquare = null;
                    }
                    operation.setApplysquare(applyaquare);
                    operation.setOperationId(rs.getString(4));
                    operation.setOperationStatus(rs.getInt(5));
                    Professor professor = new Professor();
                    professor.setId(rs.getString(6));
                    operation.setDetailDemand(rs.getString(7));
                    operation.setDeliverTime(rs.getString(8));
                    Resource resource = new Resource();
					resource.setResourceId(rs.getString(9));
                    BigDecimal dealPrice = rs.getBigDecimal(10);
                    if(rs.wasNull()){
                    	dealPrice = null;
                    }
                    operation.setDealPrice(dealPrice);
                    String dealTime = rs.getString(11);
                    if(rs.wasNull()){
                    	dealTime = null;
                    }
                    operation.setDealTime(dealTime);
                    String payMethod = rs.getString(12);
                    if(rs.wasNull()){
                    	payMethod = null;
                    }
                    operation.setPayMethod(payMethod);
                    professor.setName(rs.getString(13));
                    operation.setProfessor(professor);
					resource.setResourceName(rs.getString(14));
					Professor professor2 = new Professor();
                    professor2.setId(rs.getString(15));
                    professor2.setName(rs.getString(16));
                    resource.setProfessor(professor2);
					operation.setResource(resource);
					operations.add(operation);
                }
                return queryResult;
            }finally{
                try{rs.close();}catch(Exception e3){}
            }
        }finally{
            try{ps.close();}catch(Exception e4){}
        }
    }
	
	@SelectList
	public abstract List<Operation> queryResourceId(Connection con, String resourceId) throws SQLException;
	
	@DeleteWith
	@From(Operation.class)
	public abstract int delete(Connection con, String operationId) throws SQLException;
	
	/**
	 * 获取当前资源申请的状态值。
	 * @param con
	 * @param operationId 资源申请ID
	 * @return 返回资源申请状态值
	 * @throws SQLException
	 */
	public int queryOper(Connection con, String operationId) throws SQLException{
		String sql = "SELECT OPERATION_STATUS FROM OPERATION WHERE OPERATION_ID = ?";
		PreparedStatement ps = con.prepareStatement(sql);
		try {
			ps.setString(1, operationId);
			ResultSet rs = ps.executeQuery();
			try{
				rs.next();
				return rs.getInt(1);
			}finally{
                try{rs.close();}catch(Exception e1){}
            }
		}finally{
            try{ps.close();}catch(Exception e2){}
        }
	}
	
	public int deleteDem(Connection con,String operationId,String demandId) throws SQLException{
        int index = 1;
        String sql = "DELETE FROM OPERATION WHERE OPERATION_STATUS = 1 AND DEMAND_ID = ? AND OPERATION_ID = ?";
        PreparedStatement ps = con.prepareStatement(sql);
        try{
            ps.setString(index++,demandId);
            ps.setString(index++,operationId);
            return ps.executeUpdate();
        }finally{
            try{ps.close();}catch(Exception e){}
        }
    }
	
	/**
	 * 分页查询资源申请 多条件模糊查询
	 * @param con
	 * @param professorName 申请人姓名
	 * @param resourceName 资源名称
	 * @param firstTime 申请时间 查询初试时间
	 * @param lastTime 申请时间 查询结束时间
	 * @param lowPrice 成交价格 查询较低价格
	 * @param highPrice 成交时间 查询较高价格
	 * @param pageSize 每页记录数
	 * @param pageNo 当前页码
	 * @return
	 * @throws SQLException
	 */
	public PageQueryResult<Operation> queryPage(Connection con,String professorName,String resourceName,String firstTime,String lastTime,java.math.BigDecimal lowPrice,java.math.BigDecimal highPrice,int pageSize,int pageNo) throws SQLException{
        int total = 0;
        PageQueryResult<Operation> queryResult = new PageQueryResult<Operation>();
        int index = 1;
        boolean hasProName = null != professorName;
        boolean hasResName = null != resourceName;
        boolean hasFirstTime = null != firstTime;
        boolean hasLastTime = null != lastTime;
        boolean hasLowPrice = null != lowPrice;
        boolean hasHighPrice = null != highPrice;
        StringBuilder sql = new StringBuilder();
        boolean hasWhere = true;
        if(hasProName){
        	hasWhere =  false;
            sql.append(" WHERE DEMAND_ID (SELECT PROFESSOR.PROFESSOR.ID FROM PROFESSOR WHERE PROFESSOR.NAME LIKE ?)");
        }
        if(hasResName){
            if(hasWhere){
            	hasWhere =  false;
                sql.append(" WHERE OPERATION.RESOURCE_ID IN (SELECT RESOURCE.RESOURCE_ID FROM RESOURCE WHERE RESOURCE_NAME LIKE ?)");
            }else{
                sql.append(" OR OPERATION.RESOURCE_ID IN (SELECT RESOURCE.RESOURCE_ID FROM RESOURCE WHERE RESOURCE_NAME LIKE ?)");
            }
        }
        if(hasFirstTime){
            if(hasWhere){
            	hasWhere =  false;
                sql.append(" WHERE OPERATION.CREATE_TIME > ?");
            }else{
                sql.append(" AND OPERATION.CREATE_TIME > ?");
            }
        }
        if(hasLastTime){
            if(hasWhere){
            	hasWhere =  false;
                sql.append(" WHERE OPERATION.CREATE_TIME <= ?");
            }else{
                sql.append(" AND OPERATION.CREATE_TIME <= ?");
            }
        }
        if(hasLowPrice){
            if(hasWhere){
            	hasWhere =  false;
                sql.append(" WHERE DEAL_PRICE > ?");
            }else{
                sql.append(" AND DEAL_PRICE > ?");
            }
        }
        if(hasHighPrice){
            if(hasWhere){
            	hasWhere =  false;
                sql.append(" WHERE DEAL_PRICE <= ?");
            }else{
                sql.append(" AND DEAL_PRICE <= ?");
            }
        }
        StringBuilder whereSql = sql;
        sql = new StringBuilder();
        sql.append("SELECT COUNT(1) FROM OPERATION");
        if(whereSql.length()>0){
            sql.append(whereSql);
        }
        PreparedStatement ps = con.prepareStatement(sql.toString());
        try{
            if(hasProName){
                ps.setString(index++,professorName);
            }
            if(hasResName){
                ps.setString(index++,resourceName);
            }
            if(hasFirstTime){
                ps.setString(index++,firstTime);
            }
            if(hasLastTime){
                ps.setString(index++,lastTime);
            }
            if(hasLowPrice){
                ps.setBigDecimal(index++,lowPrice);
            }
            if(hasHighPrice){
                ps.setBigDecimal(index++,highPrice);
            }
            queryResult.setPageSize(pageSize);
            ResultSet rs = ps.executeQuery();
            try{
                rs.next();
                total = rs.getInt(1);
            }finally{
                try{rs.close();}catch(Exception e1){}
            }
        }finally{
            try{ps.close();}catch(Exception e2){}
        }
        queryResult.setTotal(total);
        if(0== total){
        	queryResult.setPageNo(1);
        	queryResult.setData(Collections.<Operation>emptyList());
            return queryResult;
        }
        index = 1;
        boolean firstPage = (1 == pageNo);
        if(firstPage){
        	queryResult.setPageNo(1);
            sql = new StringBuilder();
            sql.append("SELECT REPLY_NOTES,OPERATION.CREATE_TIME,OPERATION_ID,OPERATION_STATUS,DEMAND_ID,"
            		+ " DETAIL_DEMAND,DELIVER_TIME,OPERATION.RESOURCE_ID,DEAL_PRICE,DEAL_TIME,PAY_METHOD,"
            		+ " P1.NAME,RESOURCE_NAME,RESOURCE.PROFESSOR_ID,P2.NAME,APPLYSQUARE "
            		+ " FROM OPERATION LEFT JOIN PROFESSOR P1 ON DEMAND_ID = P1.ID "
            		+ " LEFT JOIN RESOURCE ON OPERATION.RESOURCE_ID = RESOURCE.RESOURCE_ID "
            		+ " LEFT JOIN PROFESSOR P2 ON RESOURCE.PROFESSOR_ID = P2.ID");
            if(whereSql.length()>0){
                sql.append(whereSql);
            }
            sql.append(" ORDER BY OPERATION.CREATE_TIME");
            sql.append(" LIMIT ").append(pageSize);
        }else{
            int pageNum = total / pageSize;
            if(total % pageSize != 0){
                ++pageNum;
            }
            if(pageNo > pageNum){
                pageNo = pageNum;
            }
            queryResult.setPageNo(pageNo);
            --pageNo;
            int offset = (pageNo * pageSize);
            sql = new StringBuilder();
            sql.append("SELECT REPLY_NOTES,OPERATION.CREATE_TIME,OPERATION_ID,OPERATION_STATUS,DEMAND_ID,"
            		+ " DETAIL_DEMAND,DELIVER_TIME,OPERATION.RESOURCE_ID,DEAL_PRICE,DEAL_TIME,PAY_METHOD,"
            		+ " P1.NAME,RESOURCE_NAME,RESOURCE.PROFESSOR_ID,P2.NAME,APPLYSQUARE "
            		+ " FROM OPERATION LEFT JOIN PROFESSOR P1 ON DEMAND_ID = P1.ID "
            		+ " LEFT JOIN RESOURCE ON OPERATION.RESOURCE_ID = RESOURCE.RESOURCE_ID "
            		+ " LEFT JOIN PROFESSOR P2 ON RESOURCE.PROFESSOR_ID = P2.ID");
            if(whereSql.length()>0){
                sql.append(whereSql);
            }
            sql.append(" ORDER BY OPERATION.CREATE_TIME");
            sql.append(" LIMIT ").append(pageSize).append(" OFFSET ").append(offset);
        }
        ps = con.prepareStatement(sql.toString());
        try{
            if(hasProName){
                ps.setString(index++,professorName);
            }
            if(hasResName){
                ps.setString(index++,resourceName);
            }
            if(hasFirstTime){
                ps.setString(index++,firstTime);
            }
            if(hasLastTime){
                ps.setString(index++,lastTime);
            }
            if(hasLowPrice){
                ps.setBigDecimal(index++,lowPrice);
            }
            if(hasHighPrice){
                ps.setBigDecimal(index++,highPrice);
            }
            ResultSet rs = ps.executeQuery();
            try{
                List<Operation> operations = new ArrayList<Operation>();
                queryResult.setData(operations);
                int size = 0;
                while((size<pageSize) && rs.next()){
                    ++size;
                    Operation operation =  new Operation();
                    operation.setReplyNotes(rs.getString(1));
                    operation.setCreateTime(rs.getString(2));
                    operation.setOperationId(rs.getString(3));
                    operation.setOperationStatus(rs.getInt(4));
                    Professor professor = new Professor();
                    professor.setId(rs.getString(5));
                    operation.setDetailDemand(rs.getString(6));
                    operation.setDeliverTime(rs.getString(7));
                    Resource resource = new Resource();
                    resource.setResourceId(rs.getString(8));
                    operation.setDealPrice(rs.getBigDecimal(9));
                    operation.setDealTime(rs.getString(10));
                    operation.setPayMethod(rs.getString(11));
                    professor.setName(rs.getString(12));
                    operation.setProfessor(professor);
                    resource.setResourceName(rs.getString(13));
                    Professor professor2 = new Professor();
                    professor2.setId(rs.getString(14));
                    professor2.setName(rs.getString(15));
                    resource.setProfessor(professor2);
                    operation.setResource(resource);
                    operation.setApplysquare(rs.getString(16));
                    operations.add(operation);
                }
                return queryResult;
            }finally{
                try{rs.close();}catch(Exception e3){}
            }
        }finally{
            try{ps.close();}catch(Exception e4){}
        }
    }
	
}
