package com.ekexiu.portal.service;

import java.sql.Connection;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.annotation.DefaultValue;
import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.web.annotation.Path;
import org.jfw.apt.web.annotation.operate.Get;
import org.jfw.apt.web.annotation.operate.Post;
import org.jfw.apt.web.annotation.param.JdbcConn;
import org.jfw.util.PageQueryResult;
import org.jfw.util.StringUtil;
import org.jfw.util.exception.JfwBaseException;

import com.ekexiu.portal.dao.DemandDao;
import com.ekexiu.portal.dao.DemandSubOrIndusDao;
import com.ekexiu.portal.dao.ProfessorDao;
import com.ekexiu.portal.po.Demand;
import com.ekexiu.portal.po.DemandSubOrIndus;
import com.ekexiu.portal.pojo.EditProfessor;

@Path("/demand")
public class DemandService {
	public static final String MAX_CREATETIME="9";
	
	@Autowrie
	private DemandDao demandDao;
	@Autowrie
	private DemandSubOrIndusDao demandSubOrIndusDao;
	@Autowrie
	private ProfessorDao professorDao;
	@Autowrie
	private ImageService imageService;
	@Autowrie
	private OrgService orgService;
	
	public DemandDao getDemandDao() {
		return demandDao;
	}
	public void setDemandDao(DemandDao demandDao) {
		this.demandDao = demandDao;
	}
	public DemandSubOrIndusDao getDemandSubOrIndusDao() {
		return demandSubOrIndusDao;
	}
	public void setDemandSubOrIndusDao(DemandSubOrIndusDao demandSubOrIndusDao) {
		this.demandSubOrIndusDao = demandSubOrIndusDao;
	}
	public ProfessorDao getProfessorDao() {
		return professorDao;
	}
	public void setProfessorDao(ProfessorDao professorDao) {
		this.professorDao = professorDao;
	}
	public ImageService getImageService() {
		return imageService;
	}
	public void setImageService(ImageService imageService) {
		this.imageService = imageService;
	}
	public OrgService getOrgService() {
		return orgService;
	}
	public void setOrgService(OrgService orgService) {
		this.orgService = orgService;
	}
	
	@Post
	@Path
	public void insert(@JdbcConn(true) Connection con,Demand demand,@Nullable String[] args)throws SQLException{
		String demandId = StringUtil.buildUUID();
		demand.setDemandId(demandId);
		demand.setDemandStatus("1");
		demand.setOrderKey(System.nanoTime());
		this.demandDao.insert(con, demand);
		if(args != null){
			for (int i = 0; i < args.length; i++) {
				DemandSubOrIndus subOrIndus = new DemandSubOrIndus();
				subOrIndus.setId(StringUtil.buildUUID());
				subOrIndus.setSort(i+1);
				subOrIndus.setSubOrIndus(args[i]);
				subOrIndus.setDemandId(demandId);
				this.demandSubOrIndusDao.insert(con, subOrIndus);
			}
		}
	}
	
	@Post
	@Path("/close")
	public void closeDemand(@JdbcConn(true) Connection con,String demandId)throws SQLException{
		SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
		String closeTime = sdf.format(new Date());
		//更新需求状态为关闭-0
		this.demandDao.updateDemandStatus(con, demandId, "0", closeTime);
	}
	
	@Get
	@Path("/queryOne")
	public Demand queryOne(@JdbcConn Connection con,String demandId)throws SQLException{
		Demand demand = this.demandDao.queryOne(con, demandId);
		EditProfessor professor = this.professorDao.queryBaseInfo(con, demand.getDemander());
		if(professor != null){
			professor.setHasHeadImage(this.imageService.hasProfessorImage(demand.getDemander()));
		}
		demand.setProfessor(professor);
		if(demand.getOrgId() != null){
			demand.setOrganization(this.orgService.query(con, demand.getOrgId()));
		}
		return demand;
	}
	
	/**
	 * 查询我发布中的需求总数
	 * @param con
	 * @param demander 发布者ID
	 * @return 返回状态发布中的需求总数
	 * @throws SQLException
	 * @throws JfwBaseException 
	 */
	@Get
	@Path("/queryMyPublish")
	public int queryMyPublish(@JdbcConn Connection con,String demander)throws SQLException, JfwBaseException{
		return this.demandDao.queryCountByStatus(con, demander, "1");
	}
	
	@Get
	@Path("/queryPublish")
	public int queryPublish(@JdbcConn Connection con)throws SQLException, JfwBaseException{
		return this.demandDao.queryCountStatus(con, "1");
	}
	
	@Get
	@Path("/qaForOrg")
	public List<Demand> queryForOrg(@JdbcConn Connection con,String orgId)throws SQLException{
		return this.demandDao.queryForOrg(con, orgId);
	}
	
	@Get
	@Path("/byDemander")
	public List<Demand> queryByDemander(@JdbcConn Connection con,String demander)throws SQLException{
		List<Demand> demands = this.demandDao.queryByDemander(con, demander);
		return demands;
	}
	
	@Get
	@Path("/pqDemander")
	public PageQueryResult<Demand> pageQueryByDemander(@JdbcConn Connection con,String demander,@Nullable String demandStatus,
			@DefaultValue("0") int sortType,@DefaultValue("5") int pageSize,@DefaultValue("1") int pageNo)throws SQLException{
		return this.demandDao.pageQueryByDemander(con, demander, demandStatus, sortType, pageSize, pageNo);
	}
	
	@Get
	@Path("/pq")
	public PageQueryResult<Demand> pageQuery(@JdbcConn Connection con,@Nullable String demandAim,
			@DefaultValue("0") int sortType,@DefaultValue("5") int pageSize,@DefaultValue("1") int pageNo)throws SQLException{
		PageQueryResult<Demand> queryResult = this.demandDao.pageQuery(con, "1", demandAim, sortType, pageSize, pageNo);
		List<Demand> demands = queryResult.getData();
		for (Demand demand : demands) {
			EditProfessor professor = this.professorDao.queryBaseInfo(con, demand.getDemander());
			if(professor != null){
				professor.setHasHeadImage(this.imageService.hasProfessorImage(demand.getDemander()));
			}
			demand.setProfessor(professor);
		}
		queryResult.setData(demands);
		return queryResult;
	}
	
	@Get
	@Path("/pqOrg")
	public PageQueryResult<Demand> pageQueryOrg(@JdbcConn Connection con,String orgId,@Nullable String demandStatus,
			@DefaultValue("0") int sortType,@DefaultValue("5") int pageSize,@DefaultValue("1") int pageNo)throws SQLException{
		PageQueryResult<Demand> queryResult = this.demandDao.pageQueryOrg(con, orgId, demandStatus, sortType, pageSize, pageNo);
		List<Demand> demands = queryResult.getData();
		for (Demand demand : demands) {
			EditProfessor professor = this.professorDao.queryBaseInfo(con, demand.getDemander());
			if(professor != null){
				professor.setHasHeadImage(this.imageService.hasProfessorImage(demand.getDemander()));
			}
			demand.setProfessor(professor);
		}
		queryResult.setData(demands);
		return queryResult;
	}
	
	@Get
	@Path("/ql")
	public List<Demand> queryByStatus(@JdbcConn Connection con,@Nullable String demandAim,@Nullable String subject,
			@Nullable String industry,@DefaultValue("com.ekexiu.portal.service.DemandService.MAX_CREATETIME") String createTime,
			@DefaultValue("Long.MAX_VALUE") long orderKey,@DefaultValue("20") int rows,@DefaultValue("0") int sortType)throws SQLException{
		if(subject != null) subject = "%"+subject+"%";
		if(industry != null) industry = "%"+industry+"%";
		List<Demand> demands = this.demandDao.queryByStatus(con, "1", demandAim, subject, industry, createTime, orderKey, rows, sortType);
		for (Demand demand : demands) {
			EditProfessor professor = this.professorDao.queryBaseInfo(con, demand.getDemander());
			if(professor != null){
				professor.setHasHeadImage(this.imageService.hasProfessorImage(demand.getDemander()));
			}
			demand.setProfessor(professor);
		}
		return demands;
	}

}
