package com.ekexiu.portal.service;

import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.List;

import javax.imageio.ImageIO;

import org.jfw.util.codec.Base64;
import org.jfw.util.exception.JfwBaseException;
import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.web.annotation.Path;
import org.jfw.apt.web.annotation.operate.Delete;
import org.jfw.apt.web.annotation.operate.Get;
import org.jfw.apt.web.annotation.operate.Post;
import org.jfw.apt.web.annotation.operate.Put;
import org.jfw.apt.web.annotation.param.JdbcConn;
import org.jfw.apt.web.annotation.param.PathVar;
import org.jfw.util.JpgUtil;
import org.jfw.util.StringUtil;
import org.jfw.util.io.IoUtil;

import com.ekexiu.portal.dao.ImageDao;
import com.ekexiu.portal.po.Image;
import com.ekexiu.portal.util.PictureRotating;

@Path("/images")
public class ImagesService {
	private File tmpPath;
	private File resourcePath;
	private int resImageMaxLen = 240;
	private static final String JPG = "jpg";
	@Autowrie
	private ImageDao imageDao;
	
	public File getTmpPath() {
		return tmpPath;
	}

	public void setTmpPath(File tmpPath) {
		this.tmpPath = tmpPath;
	}

	public File getResourcePath() {
		return resourcePath;
	}

	public void setResourcePath(File resourcePath) {
		this.resourcePath = resourcePath;
	}

	public int getResImageMaxLen() {
		return resImageMaxLen;
	}

	public void setResImageMaxLen(int resImageMaxLen) {
		this.resImageMaxLen = resImageMaxLen;
	}

	public ImageDao getImageDao() {
		return imageDao;
	}

	public void setImageDao(ImageDao imageDao) {
		this.imageDao = imageDao;
	}
	
	private byte[] resImage(byte[] src, int maxLen) throws IOException {
		ByteArrayInputStream in = new ByteArrayInputStream(src);
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		JpgUtil.scalingZoom(in, out, maxLen);
		out.flush();
		return out.toByteArray();
	}
	
	private byte[] readTmpFile(String fn) throws JfwBaseException {
		File file = new File(this.tmpPath, fn);
		if (!file.exists())
			throw new JfwBaseException(90, "resource image not exists");
		try {
			InputStream in = new FileInputStream(file);
			ByteArrayOutputStream out = new ByteArrayOutputStream();
			IoUtil.copy(in, out, true, true);
			return out.toByteArray();
		} catch (IOException e) {
			throw new JfwBaseException(91, "read temp resource image error", e);
		}
	}
	
	@Post
	@Path("/resourceImg")
	public String insert(@JdbcConn(true) Connection con, String resourceId, String fn) 
			throws SQLException, IOException, JfwBaseException {
		byte[] src = this.readTmpFile(fn);
		src = JpgUtil.read(src);
		byte[] shareResImage = this.resImage(src, this.resImageMaxLen);
		IoUtil.saveStream(new FileOutputStream(new File(this.resourcePath, resourceId + "." + JPG)), src, true);
		IoUtil.saveStream(new FileOutputStream(new File(this.resourcePath, resourceId + "_s." + JPG)), shareResImage, true);
		Image image1 = this.imageDao.queryOne(con, resourceId);
		if(null == image1){
			Image image = new Image();
			String imageId = StringUtil.buildUUID();
			image.setImageId(imageId);
			String imageSrc = this.resourcePath + "/" + resourceId + "." + JPG;
			image.setImageSrc(imageSrc);
			image.setResourceId(resourceId);
			this.imageDao.insert(con, image);
			return imageId;
		} else {
			return null;
		}
	}
	
	@Post
	@Path("/saveResImg")
	public String insertResourceImg(@JdbcConn(true) Connection con, String resourceId, String base64, Integer angle) throws SQLException, IOException {
		Base64 bs64 = new Base64();
		byte[] bs = bs64.decode(base64.getBytes("UTF-8"));
		String imageSrc = this.resourcePath + "/" + resourceId + "." + JPG;
		String resImage = this.resourcePath + "/" + resourceId + "_s." + JPG;
		FileOutputStream fos = new FileOutputStream(imageSrc);
		FileOutputStream fos2 = new FileOutputStream(resImage);
		try {
			if(0 == angle){
				byte[] bytes = resImage(bs, this.resImageMaxLen);
				fos.write(bs);
				fos2.write(bytes);
			}else{
				java.awt.Image image = PictureRotating.bytesToBufferedImage(bs);
				BufferedImage des = PictureRotating.Rotate(image, angle);
				ByteArrayOutputStream out = new ByteArrayOutputStream();
				@SuppressWarnings("unused")
				boolean flag = ImageIO.write(des, JPG, out);
				byte[] bytes = out.toByteArray();
				byte[] bs2 = resImage(bytes, this.resImageMaxLen);
				fos.write(bytes);
				fos2.write(bs2);
			}
			fos.flush();
			fos2.flush();
		} finally {
			fos.close();
			fos2.close();
		}
		Image image1 = this.imageDao.queryOne(con, resourceId);
		if(null == image1){
			Image image = new Image();
			String imageId = StringUtil.buildUUID();
			image.setImageId(imageId);
			image.setImageSrc(imageSrc);
			image.setResourceId(resourceId);
			this.imageDao.insert(con, image);
			return imageId;
		} else {
			return null;
		}
	}
	
	@Put
	@Path
	public void update(@JdbcConn(true) Connection con, String resourceId) throws SQLException{
		
	}
	
	@Get
	@Path("/{imageId}")
	public Image query(@JdbcConn Connection con, @PathVar String imageId) throws SQLException {
		return this.imageDao.query(con, imageId);
	}
	
	@Get
	@Path("/qa")
	public List<Image> queryAll(@JdbcConn Connection con) throws SQLException {
		return this.imageDao.query(con);		
	}
	
	@Get
	@Path("/qares")
	public List<Image> queryRes(@JdbcConn Connection con, String resourceId) throws SQLException {
		return this.imageDao.queryRes(con, resourceId);
	}
	
	@Delete
	@Path("/{imageId}")
	public void deleteId(@JdbcConn(true) Connection con, @PathVar String imageId) throws SQLException {
		this.imageDao.deleteId(con, imageId);
	}
	
	@Delete
	@Path("/src")
	public void deleteSrc(@JdbcConn(true) Connection con, String imageSrc) throws SQLException {
		this.imageDao.deleteSrc(con, imageSrc);
	}
	
	@Delete
	@Path("/del")
	public void deleteRes(@JdbcConn(true) Connection con, String resourceId) throws SQLException {
		this.imageDao.deleteRes(con, resourceId);
	}
}
