package com.ekexiu.portal.service;

import java.io.IOException;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.List;

import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.annotation.DefaultValue;
import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.web.annotation.Path;
import org.jfw.apt.web.annotation.operate.Get;
import org.jfw.apt.web.annotation.operate.Post;
import org.jfw.apt.web.annotation.param.JdbcConn;
import org.jfw.apt.web.annotation.param.PathVar;
import org.jfw.util.StringUtil;

import com.ekexiu.portal.dao.OrgDao;
import com.ekexiu.portal.po.Organization;
import com.ekexiu.portal.pojo.EditOrganization;

@Path("/org")
public class OrgService {
	
	private String defaultOrgType;
	/**
	 * 默认企业认证状态为 0-未认证
	 */
	private String defaultAuthStatus = "0";
	
	@Autowrie
	private ImageService imageService;
	@Autowrie
	private OrgDao orgDao;
	
	public ImageService getImageService() {
		return imageService;
	}

	public void setImageService(ImageService imageService) {
		this.imageService = imageService;
	}

	public OrgDao getOrgDao() {
		return orgDao;
	}

	public void setOrgDao(OrgDao orgDao) {
		this.orgDao = orgDao;
	}

	public String getDefaultOrgType() {
		return defaultOrgType;
	}

	public void setDefaultOrgType(String defaultOrgType) {
		this.defaultOrgType = defaultOrgType;
	}

	public String getDefaultAuthStatus() {
		return defaultAuthStatus;
	}

	public void setDefaultAuthStatus(String defaultAuthStatus) {
		this.defaultAuthStatus = defaultAuthStatus;
	}

	@Post
	@Path
	public void insert(@JdbcConn(true) Connection con,Organization organization) throws SQLException 
	{
		organization.setId(StringUtil.buildUUID());
		if(organization.getOrgType()== null) organization.setOrgType(this.defaultOrgType);
		this.orgDao.insert(con, organization);
	}
	
	
	@Post
	@Path("/update")
	public void update(@JdbcConn(true) Connection con,Organization organization,@Nullable String fn) throws SQLException, IOException{
		if(fn != null){
			this.imageService.saveOrgLogo(organization.getId(), fn);
		}
		this.orgDao.update(con, organization);
	}
	
	@Get
	@Path("/{id}")
	public EditOrganization query(@JdbcConn(false) Connection con ,@PathVar String id)throws SQLException{
		EditOrganization editOrg = this.orgDao.queryEditOrg(con, id);
		editOrg.setHasOrgLogo(this.imageService.hasOrgLogo(id));
		return editOrg;
	}
	
	@Get
	@Path("/authStatus")
	public String queryAuthStatus(@JdbcConn Connection con,String id)throws SQLException{
		return this.orgDao.queryAuthStatus(con, id);
	}
	
	@Get
	@Path("/qa")
	public List<Organization> query(@JdbcConn(false) Connection con )throws SQLException{
		return this.orgDao.query(con);
	}
	@Get
	@Path("/lq")
	public List<Organization> query(@JdbcConn Connection con ,@Nullable String name,@DefaultValue("5") int rows) throws SQLException
	{
		if(name==null) name = "%";
		else name="%"+name+"%";
		return this.orgDao.limitQuery(con, name, rows);
	}
	public String createOrganization(Connection con, String orgName) throws SQLException, IOException {
		String id = StringUtil.buildUUID();
		Organization org = new Organization();
		org.setId(id);
		org.setName(orgName);
		org.setOrgType(this.defaultOrgType);
		org.setAuthStatus(this.defaultAuthStatus);
		try {
			orgDao.insert(con, org);
			this.imageService.saveDefaultOrgLogo(id);
		} catch (SQLException e) {
			if ("23505".equals(e.getSQLState())) {
				con.rollback();
				id = orgDao.queryByName(con, orgName);
				if (id == null)
					throw new RuntimeException("not fonud  with query organization by name");
				return id;
			}
			throw e;
		}
		return id;
	}
	
}
